@@ -20,7 +20,7 @@ Debian::Packages::Compare - emdebian repository comparison support
 
 =head1 VERSION
 
-Version 0.2.0
+Version 3.0.0
 
 =cut
 
@@ -30,9 +30,10 @@ use vars qw(@ISA @EXPORT);
  get_missing_sources get_missing_binaries get_britney_list
  get_single_package get_archlist get_locale_roots get_base
  get_components get_britney_complaint get_missing_builddeps
- get_suite_names get_unfrozen_suite_names );
+ get_suite_names get_unfrozen_suite_names get_package_cmpnt
+ get_unreleased_architectures );
 
-our $OURVERSION = "0.2.3";
+our $OURVERSION = "3.0.0";
 
 =head1 Synopsis
 
@@ -93,8 +94,7 @@ Reads the supported architectures and returns a list.
 
 =cut
 
-sub get_archlist
-{
+sub get_archlist {
 	my (@stanzas, $parser, $conf, $stanza);
 	my ($suite, $repo) = @_;
 	my @archlist=();
@@ -105,11 +105,9 @@ sub get_archlist
 	return undef unless (-f $input);
 	$conf = $parser->parse_file("$input", {'stripComments' => 'true'});
 	@stanzas = @$conf;
-	foreach $stanza (@stanzas)
-	{
+	foreach $stanza (@stanzas) {
 		if (($$stanza{'Codename'} eq "$suite")
-			or ($$stanza{'Suite'} eq "$suite"))
-		{
+			or ($$stanza{'Suite'} eq "$suite")) {
 			my $line = $$stanza{'Architectures'};
 			@archlist = split(' ', $line);
 		}
@@ -126,8 +124,7 @@ Reads the supported locale roots and returns a list.
 
 =cut
 
-sub get_locale_roots
-{
+sub get_locale_roots {
 	my (@stanzas, $parser, $conf, $stanza);
 	my ($suite, $repo) = @_;
 	my @locroots=();
@@ -138,11 +135,9 @@ sub get_locale_roots
 	return undef unless (-f $input);
 	$conf = $parser->parse_file("$input", {'stripComments' => 'true'});
 	@stanzas = @$conf;
-	foreach $stanza (@stanzas)
-	{
+	foreach $stanza (@stanzas) {
 		if (($$stanza{'Codename'} eq "$suite")
-			or ($$stanza{'Suite'} eq "$suite"))
-		{
+			or ($$stanza{'Suite'} eq "$suite")) {
 			my $line = $$stanza{'Components'};
 			@locroots = split(' ', $line);
 		}
@@ -159,8 +154,7 @@ likely get confused results with this kind of support.
 
 =cut
 
-sub get_components
-{
+sub get_components {
 	my (@stanzas, $parser, $conf, $stanza);
 	my ($suite, $repo) = @_;
 	my @cmpnts=();
@@ -171,11 +165,9 @@ sub get_components
 	return undef unless (-f $input);
 	$conf = $parser->parse_file("$input", {'stripComments' => 'true'});
 	@stanzas = @$conf;
-	foreach $stanza (@stanzas)
-	{
+	foreach $stanza (@stanzas) {
 		if (($$stanza{'Codename'} eq "$suite")
-			or ($$stanza{'Suite'} eq "$suite"))
-		{
+			or ($$stanza{'Suite'} eq "$suite")) {
 			my $line = $$stanza{'Components'};
 			@cmpnts = split(' ', $line);
 		}
@@ -183,6 +175,38 @@ sub get_components
 	return \@cmpnts;
 }
 
+sub get_unreleased_architectures {
+	my (@stanzas, $parser, $conf, $stanza);
+	my $repo = shift;
+	my @temp;
+	my @unreleased=();
+	my @candidates=();
+	return undef unless defined $base;
+	$parser = new Parse::DebControl;
+	my $input = "$base/$repo/conf/distributions";
+	$input =~ s://:/:g;
+	return undef unless (-f $input);
+	$conf = $parser->parse_file("$input", {'stripComments' => 'true'});
+	@stanzas = @$conf;
+	foreach $stanza (@stanzas) {
+		if (($$stanza{'Codename'} eq "unstable")
+			or ($$stanza{'Suite'} eq "unstable")) {
+			my $line = $$stanza{'Architectures'};
+			@temp = split(' ', $line);
+		}
+		if (($$stanza{'Codename'} eq "testing")
+			or ($$stanza{'Suite'} eq "testing")) {
+			my $line = $$stanza{'Architectures'};
+			@candidates = split(' ', $line);
+		}
+	}
+	foreach my $c (@temp) {
+		push (@unreleased, $c)
+			if (scalar (grep(/$c/, @candidates)) == 0);
+	}
+	return \@unreleased;
+}
+
 =head1 get_suite_names
 
 Parses the conf/distributions file for the specified repository and
@@ -191,8 +215,7 @@ of the Suite: and Codename: variables in conf/distributions.
 
 =cut
 
-sub get_suite_names
-{
+sub get_suite_names {
 	my (@stanzas, $parser, $conf, $stanza);
 	my $repo = shift;
 	my %suites=();
@@ -203,8 +226,7 @@ sub get_suite_names
 	return undef unless (-f $input);
 	$conf = $parser->parse_file("$input", {'stripComments' => 'true'});
 	@stanzas = @$conf;
-	foreach $stanza (@stanzas)
-	{
+	foreach $stanza (@stanzas) {
 		$suites{$$stanza{'Codename'}}++;
 		$suites{$$stanza{'Suite'}}++;
 	}
@@ -220,8 +242,7 @@ Codename: variables are NOT included.
 
 =cut
 
-sub get_unfrozen_suite_names
-{
+sub get_unfrozen_suite_names {
 	my (@stanzas, $parser, $conf, $stanza);
 	my $repo = shift;
 	my %suites=();
@@ -232,10 +253,8 @@ sub get_unfrozen_suite_names
 	return undef unless (-f $input);
 	$conf = $parser->parse_file("$input", {'stripComments' => 'true'});
 	@stanzas = @$conf;
-	foreach $stanza (@stanzas)
-	{
-		if (exists $$stanza{'ReadOnly'})
-		{
+	foreach $stanza (@stanzas) {
+		if (exists $$stanza{'ReadOnly'}) {
 			next if ($$stanza{'ReadOnly'} eq 'Yes');
 		}
 		$suites{$$stanza{'Suite'}}++;
@@ -255,13 +274,11 @@ a single base directory:
 
 =cut
 
-sub set_base
-{
+sub set_base {
 	$base = shift;
 }
 
-sub get_base
-{
+sub get_base {
 	return $base;
 }
 
@@ -272,39 +289,38 @@ file. Not usually called directly.
 
 =cut
 
-sub read_sources
-{
+sub read_sources {
 	my ($suite, $repo) = @_;
-	if (not defined $base)
-	{
+	if (not defined $base) {
 		print "Base directory has not been set.";
 		return undef;
 	}
 	my %package=();
 	my %list=();
-	# Sources are not split into components at this time.
-	my $input = "$base/$repo/dists/$suite/main/source/Sources.gz";
-	$input =~ s://:/:g;
-	if (not -f $input)
-	{
-		print "Cannot find Sources.gz: $input\n";
-		return undef;
-	}
-	my $z = new IO::Uncompress::Gunzip $input
-			or die "IO::Uncompress::Gunzip failed: $GunzipError\n";
-	my $parser = Parse::Debian::Packages->new( $z );
-	while (%package = $parser->next)
-	{
-		$list{$package{'Package'}}{'source'}=$package{'Version'};
-		my $src = (not defined $package{'Source'}) ?
-			$package{'Package'} :
-			$package{'Source'};
-		$src =~ s/\(.*\)//g;
-		$src =~ s/ //g;
-		$list{$package{'Package'}}{'Src'}=$src;
-		my $dep = $package{'Build-Depends'};
-		chomp ($dep) if (defined $dep);
-		$list{$package{'Package'}}{'Build-Depends'}=$dep;
+	# Support sources in multiple components
+	my $clist = get_components ($suite, $repo);
+	foreach my $cmp (@$clist) {
+		my $input = "$base/$repo/dists/$suite/$cmp/source/Sources.gz";
+		$input =~ s://:/:g;
+		if (not -f $input) {
+			print "Cannot find Sources.gz: $input\n";
+			return undef;
+		}
+		my $z = new IO::Uncompress::Gunzip $input
+				or die "IO::Uncompress::Gunzip failed: $GunzipError\n";
+		my $parser = Parse::Debian::Packages->new( $z );
+		while (%package = $parser->next) {
+			$list{$package{'Package'}}{'source'}=$package{'Version'};
+			my $src = (not defined $package{'Source'}) ?
+				$package{'Package'} :
+				$package{'Source'};
+			$src =~ s/\(.*\)//g;
+			$src =~ s/ //g;
+			$list{$package{'Package'}}{'Src'}=$src;
+			my $dep = $package{'Build-Depends'};
+			chomp ($dep) if (defined $dep);
+			$list{$package{'Package'}}{'Build-Depends'}=$dep;
+		}
 	}
 	return \%list;
 }
@@ -328,13 +344,11 @@ e.g. for the Grip repository:
 
 =cut
 
-sub read_packages
-{
+sub read_packages {
 	my ($suite, $repo) = @_;
 	return undef unless defined $base;
 	my $src = (read_sources ($suite, $repo));
-	if (not defined $src)
-	{
+	if (not defined $src) {
 		warn ("No sources found in '$repo' repository.\n");
 		return undef;
 	}
@@ -342,26 +356,22 @@ sub read_packages
 	undef $src;
 	$src = get_archlist ($suite, $repo);
 	my @archlist = @$src;
-	foreach my $arch (@archlist)
-	{
+	foreach my $arch (@archlist) {
 		next if ($arch eq 'source');
 		my %package=();
 		my ($parser, $fh);
 		# support components other than main.
 		my $c = get_components ($suite, $repo);
-		foreach my $cmpnt (@$c)
-		{
+		foreach my $cmpnt (@$c) {
 			my $file = "$base/$repo/dists/$suite/$cmpnt/binary-${arch}/Packages";
 			$file =~ s://:/:g;
-			if (not -f $file)
-			{
+			if (not -f $file) {
 				warn ("Cannot find Packages file: '$file'.\n");
 				next;
 			}
 			$fh = IO::File->new("$file") or die "$!\n";
 			$parser = Parse::Debian::Packages->new( $fh );
-			while (%package = $parser->next)
-			{
+			while (%package = $parser->next) {
 				$list{$package{'Package'}}{"$arch"}=$package{'Version'};
 				my $src = (not defined $package{'Source'}) ?
 					$package{'Package'} :
@@ -399,8 +409,7 @@ e.g.
 
 =cut
 
-sub read_locale
-{
+sub read_locale {
 	my ($suite, $repo) = @_;
 	return undef unless defined $base;
 	my $src = (read_sources ($suite, $repo));
@@ -415,24 +424,20 @@ sub read_locale
 	undef $src;
 	return undef unless (@archlist);
 	return undef unless (@locroots);
-	foreach my $cmpnt (@locroots)
-	{
-		foreach my $arch (@archlist)
-		{
+	foreach my $cmpnt (@locroots) {
+		foreach my $arch (@archlist) {
 			next if ($arch eq 'source');
 			my %package=();
 			my ($parser, $fh);
 			my $file = "$base/$repo/dists/$suite/$cmpnt/binary-${arch}/Packages";
 			$file =~ s://:/:g;
-			if (not -f $file)
-			{
+			if (not -f $file) {
 				warn ("Cannot find Packages file: '$file'.\n");
 				next;
 			}
 			$fh = IO::File->new("$file");
 			$parser = Parse::Debian::Packages->new( $fh );
-			while (%package = $parser->next)
-			{
+			while (%package = $parser->next) {
 				$list{$package{'Package'}}{"$arch"}=$package{'Version'};
 				$list{$package{'Package'}}{'locale'}=$cmpnt;
 				$list{$package{'Package'}}{'source'}=$package{'Source'}
@@ -456,28 +461,18 @@ sub get_single_package
 	my ($suite, $repo, $pkg, $arch) = @_;
 	my ($parser, $fh, %list, %package);
 	return undef unless defined $base;
-	if ($arch ne 'source')
-	{
+	if ($arch ne 'source') {
 		# support components other than main.
 		my $c = get_components ($suite, $repo);
-		foreach my $cmpnt (@$c)
-		{
+		foreach my $cmpnt (@$c) {
 			my $file = "$base/$repo/dists/$suite/$cmpnt/binary-${arch}/Packages";
 			$file =~ s://:/:g;
 			next if (not -f $file);
 			$fh = IO::File->new("$file") or die "$!\n";
 			$parser = Parse::Debian::Packages->new( $fh );
-			while (%package = $parser->next)
-			{
-				if ($package{'Package'} eq $pkg)
-				{
-					if ($cmpnt eq 'main')
-					{
-						%list = (%package);
-						next;
-					}
-					my %detail = (%package);
-					$list{$cmpnt} = \%detail;
+			while (%package = $parser->next) {
+				if ($package{'Package'} eq $pkg) {
+					%list = (%package);
 				}
 			}
 			$fh->close;
@@ -487,22 +482,51 @@ sub get_single_package
 	}
 	# support components other than main.
 	my $c = get_components ($suite, $repo);
-	foreach my $cmpnt (@$c)
-	{
+	foreach my $cmpnt (@$c) {
 		my $input = "$base/$repo/dists/$suite/$cmpnt/source/Sources.gz";
 		$input =~ s://:/:g;
 		next unless (-f $input);
 		my $z = new IO::Uncompress::Gunzip $input
 				or die "IO::Uncompress::Gunzip failed: $GunzipError\n";
 		$parser = Parse::Debian::Packages->new( $z );
-		while (%package = $parser->next)
-		{
+		while (%package = $parser->next) {
 			%list = (%package) if ($package{'Package'} eq $pkg);
 		}
 	}
 	return \%list;
 }
 
+sub get_package_cmpnt {
+	my ($suite, $repo, $pkg, $arch, $cmpnt) = @_;
+	my ($parser, $fh, %list, %package);
+	return undef unless defined $base;
+	if ($arch ne 'source') {
+		my $file = "$base/$repo/dists/$suite/$cmpnt/binary-${arch}/Packages";
+		$file =~ s://:/:g;
+		next if (not -f $file);
+		$fh = IO::File->new("$file") or die "$!\n";
+		$parser = Parse::Debian::Packages->new( $fh );
+		while (%package = $parser->next) {
+			if ($package{'Package'} eq $pkg) {
+				%list = (%package);
+			}
+		}
+		$fh->close;
+		undef $fh;
+		return \%list;
+	}
+	my $input = "$base/$repo/dists/$suite/$cmpnt/source/Sources.gz";
+	$input =~ s://:/:g;
+	next unless (-f $input);
+	my $z = new IO::Uncompress::Gunzip $input
+			or die "IO::Uncompress::Gunzip failed: $GunzipError\n";
+	$parser = Parse::Debian::Packages->new( $z );
+	while (%package = $parser->next) {
+		%list = (%package) if ($package{'Package'} eq $pkg);
+	}
+	return \%list;
+}
+
 =head1 get_missing_sources
 
 Compare two repositories for the same suite and return a list of
@@ -513,8 +537,7 @@ TODO: need a get_outdated_sources and get_outdated_binaries too.
 
 =cut
 
-sub get_missing_sources
-{
+sub get_missing_sources {
 	my ($suite, $repo1, $repo2) = @_;
 	return undef unless (defined $suite and defined $base and
 		defined $repo1 and defined $repo2);
@@ -524,17 +547,13 @@ sub get_missing_sources
 	$src = (read_sources ($suite, $repo2));
 	my %comp =%$src;
 	my %list=();
-
-# BUG: needs to look for old sources as well as missing ones.
-
-	foreach my $pkg (sort keys %orig)
-	{
+	# BUG: needs to look for old sources as well as missing ones.
+	foreach my $pkg (sort keys %orig) {
 		# convert a binary to a source to ensure each binary
 		# has a source package
 		my $src = $orig{$pkg}{'Src'};
 		next if (not defined $orig{$pkg}{'source'});
-		if (not defined $comp{$pkg}{'source'})
-		{
+		if (not defined $comp{$pkg}{'source'}) {
 			$list{$pkg} = $orig{$pkg};
 			next;
 		}
@@ -542,8 +561,7 @@ sub get_missing_sources
 	return \%list;
 }
 
-sub get_missing_builddeps
-{
+sub get_missing_builddeps {
 	my ($suite, $repo1, $repo2) = @_;
 	return undef unless (defined $suite and defined $base and
 		defined $repo1 and defined $repo2);
@@ -555,18 +573,14 @@ sub get_missing_builddeps
 	$src = (read_packages ($suite, $repo1));
 	my %bin = %$src;
 	my %list=();
-	foreach my $pkg (sort keys %orig)
-	{
+	foreach my $pkg (sort keys %orig) {
 		my $deps = $orig{$pkg}{'Build-Depends'};
 		next if not defined ($deps);
 		my @d = split(",", $deps);
-		foreach my $dd (sort @d)
-		{
-			if ($dd =~ /\|/)
-			{
+		foreach my $dd (sort @d) {
+			if ($dd =~ /\|/) {
 				my @ors = split('\|', $dd);
-				foreach my $or (@ors)
-				{
+				foreach my $or (@ors) {
 					$or =~ s/^ +//;
 					$or =~ s/ +$//;
 					$or =~ s/ *\[.*\] *//;
@@ -602,8 +616,7 @@ comparison.
 
 =cut
 
-sub get_missing_binaries
-{
+sub get_missing_binaries {
 	my ($suite, $repo1, $repo2) = @_;
 	return undef unless (defined $suite and defined $base and
 		defined $repo1 and defined $repo2);
@@ -620,10 +633,8 @@ sub get_missing_binaries
 	$src = get_archlist($suite, $repo1);
 	my @archlist = @$src;
 	my %list=();
-	foreach my $pkg (sort keys %orig)
-	{
-		foreach my $arch (@archlist)
-		{
+	foreach my $pkg (sort keys %orig) {
+		foreach my $arch (@archlist) {
 			next if ($arch eq 'source');
 			$orig_vers = $orig{$pkg}{$arch};
 			# probably a source-only package.
@@ -672,8 +683,7 @@ been called.
 
 =cut
 
-sub get_britney_list
-{
+sub get_britney_list {
 	my $str;
 	%complain=();
 	my ($repo1, $repo2) = @_;
@@ -702,11 +712,12 @@ sub get_britney_list
 	my @archlist = @$src;
 	undef $src;
 	my %list=();
-	foreach my $pkg (sort keys %origunstable)
-	{
+	my $unreleased = &get_unreleased_architectures ($repo2);
+	foreach my $pkg (sort keys %origunstable) {
 		my $skip;
-		foreach my $arch (@archlist)
-		{
+		foreach my $arch (@archlist) {
+			# skip unreleased architectures
+			next if (scalar(grep(/$arch/, @$unreleased)) != 0);
 			# check criterion 1.
 			next if (not defined $origtesting{$pkg}{$arch});
 			# skip source-only packages
@@ -719,8 +730,7 @@ sub get_britney_list
 		my $src= $origunstable{$pkg}{'Src'};
 		chomp ($src);
 		$src =~ s/ //g;
-		if (not defined $compunstable{$src}{'source'})
-		{
+		if (not defined $compunstable{$src}{'source'}) {
 			$complain{$src}++;
 			$str = "ERR: Cannot find a source package '$src' for $pkg ".
 			"in Grip unstable.\n";
@@ -729,49 +739,42 @@ sub get_britney_list
 			next;
 		}
 		$skip++ if ($origunstable{$src}{'source'} ne $compunstable{$src}{'source'});
-		if (defined $skip)
-		{
+		if (defined $skip) {
 			$complain{$src}++;
 			print "$src (source) is out of date in $repo2 unstable: " .
 				"$origunstable{$src}{'source'} ne $compunstable{$src}{'source'}\n";
 			next;
 		}
 		undef $skip;
-		foreach my $arch (@archlist)
-		{
+		foreach my $arch (@archlist) {
 			# criterion 3.
 			# check the actual result with dpkg --compare-versions
 			# retain any Emdebian version suffixes - comparing within
 			# the same repo
 			next if (not defined $compunstable{$pkg}{$arch});
-			if (not defined $comptesting{$pkg}{$arch})
-			{
+			if (not defined $comptesting{$pkg}{$arch}) {
 				$list{$pkg}{$arch}{'unstable'} = $compunstable{$pkg}{$arch};
 				next;
 			}
 			$retval = system ("dpkg --compare-versions ".
 				$compunstable{$pkg}{$arch}." '>>' ".$comptesting{$pkg}{$arch});
-			if ($retval == 0)
-			{
+			if ($retval == 0) {
 				$list{$pkg}{$arch}{'unstable'} = $compunstable{$pkg}{$arch};
 				$list{$pkg}{$arch}{'testing'} = $comptesting{$pkg}{$arch};
 			}
 			$retval /= 256;
-			if ($retval == 1)
-			{
+			if ($retval == 1) {
 				$retval = system ("dpkg --compare-versions ".
 					$compunstable{$pkg}{$arch}." '>=' ".
 					$comptesting{$pkg}{$arch});
-				if ($retval == 1)
-				{
+				if ($retval == 1) {
 					$list{$pkg}{$arch}{'unstable'} = $compunstable{$pkg}{$arch};
 					$list{$pkg}{$arch}{'testing'} = $comptesting{$pkg}{$arch};
 				}
 			}
 		}
 	}
-	if (scalar (keys %complain) > 0)
-	{
+	if (scalar (keys %complain) > 0) {
 		$str = (scalar (keys %complain) == 1) ?
 			"\tAdd the missing source package explicitly ".
 				"using\n\tem_autogrip -b $base -s ":
@@ -803,8 +806,7 @@ If there are no complaints, also returns undef.
 
 =cut
 
-sub get_britney_complaint
-{
+sub get_britney_complaint {
 	return undef if (not defined %complain);
 	return undef if (scalar (keys %complain) == 0);
 	my @c = (sort keys %complain);
